import phonenumbers

__all__ = [
    'humanize_tel',
    'safe_tel'
]


# Functions

def humanize_tel(tel, country='GB'):
    """Return a human friendly (UK format) telephone number"""

    try:
        return phonenumbers.format_number(
            phonenumbers.parse(tel, country),
            phonenumbers.PhoneNumberFormat.NATIONAL
        )
    except phonenumbers.phonenumberutil.NumberParseException:
        return None

def safe_tel(tel, country='GB'):
    """
    Used to provide a version that can safely be used in a `tel:` HTML link.
    """

    try:
        return phonenumbers.format_number(
            phonenumbers.parse(tel, country),
            phonenumbers.PhoneNumberFormat.E164
        )
    except phonenumbers.phonenumberutil.NumberParseException:
        return None



import unicodedata
from urllib.parse import parse_qs, urlencode, urlparse

from jinja2.utils import escape as html_escape, urlize
from slugify import slugify as awesome_slugify
from inflection import humanize, titleize

__all__ = [
    'humanize_bytes'
]


def humanize_bytes(number):
    """
    Return a human readable version for the given number of bytes.

    Original source: (Fred Cirera) https://stackoverflow.com/a/1094933/4657956
    """

    for unit in ['','kb','mb','gb','tb','pb','rb','zb']:

        if abs(number) < 1024.0:
            return '%3.1f%s' % (number, unit)

        number /= 1024.0

    return '%.1f%s' % (number, 'yb')


import datetime

from freezegun import freeze_time

from manhattan import formatters


def test_humanize_date():
    """Return a human readable formatted date"""

    # Check that date and datetime instances are supported
    dt = datetime.datetime(2016, 1, 1)
    assert formatters.chrono.humanize_date(dt.date()) == '1st January 2016'
    assert formatters.chrono.humanize_date(dt) == '1st January 2016'

    # Check that a date string in the format YYYY-MM-DD is supported
    assert formatters.chrono.humanize_date('2016-01-01') == '1st January 2016'

def test_humanize_date_range():
    """Return a human readable date range"""

    # Check format of 2 identical dates
    d = datetime.date(2016, 1, 1)
    assert formatters.chrono.humanize_date_range(d, d) == '1st January 2016'

    # Check format of 2 in the same month
    d1 = datetime.date(2016, 1, 1)
    d2 = datetime.date(2016, 1, 15)
    assert formatters.chrono.humanize_date_range(d1, d2) \
            == '1st - 15th January 2016'

    # Check format of 2 dates in different months
    d1 = datetime.date(2016, 1, 1)
    d2 = datetime.date(2016, 8, 1)
    assert formatters.chrono.humanize_date_range(d1, d2) \
            == '1st January - 1st August 2016'

    # Check format of 2 dates in different years
    d1 = datetime.date(2016, 1, 1)
    d2 = datetime.date(2018, 1, 1)
    assert formatters.chrono.humanize_date_range(d1, d2) == \
            '1st January 2016 - 1st January 2018'

    # Check format of 2 dates in different years where the first date is after
    # the second.
    d1 = datetime.date(2017, 1, 1)
    d2 = datetime.date(2016, 1, 1)
    assert formatters.chrono.humanize_date_range(d1, d2) == \
            '1st January 2017 - 1st January 2016'

def test_humanize_datetime():
    """Return a human readable date/time"""

    # Check datetime is supported
    dt = datetime.datetime(2016, 1, 1, 9, 15)
    assert formatters.chrono.humanize_datetime(dt) \
            == '1st January 2016 at 09:15'

    # Check a date/time string in the format YYYY-MM-DD HH:MM:SS is supported
    assert formatters.chrono.humanize_datetime('2016-01-01 09:15:30') == \
            '1st January 2016 at 09:15'

    # Check format is correct for a custom template
    assert formatters.chrono.humanize_datetime(dt, '{time} on {date}') == \
            '09:15 on 1st January 2016'

def test_humanize_duration():
    """Return a human readable duration for a given number of minutes"""

    # Check format for less than 60 minutes (minutes only)
    assert formatters.chrono.humanize_duration(5) == '5m'
    assert formatters.chrono.humanize_duration(5, abbr=False) == '5 minutes'

    # Check format for multiple of 60 (hours only)
    assert formatters.chrono.humanize_duration(120) == '2hr'
    assert formatters.chrono.humanize_duration(120, abbr=False) == '2 hours'

    # Check format for non multiple greater than 60 (hours and minutes)
    assert formatters.chrono.humanize_duration(150) == '2hr 30m'
    assert formatters.chrono.humanize_duration(150, abbr=False) \
            == '2 hours 30 minutes'

def test_humanize_time():
    """Return a human readable time"""

    # Check datetime and time are supported
    dt = datetime.datetime(2017, 1, 1, 9, 15)
    assert formatters.chrono.humanize_time(dt) == '09:15'
    assert formatters.chrono.humanize_time(dt.time()) == '09:15'

@freeze_time('2016-09-01 09:15:00')
def humanize_timediff():
    """
    Return a humanized time difference between now and the specified datetime.
    """

    # Check 5 minutes from now
    d = datetime.datetime(2016, 9, 1, 9, 20)
    assert formatters.chrono.humanize_timedelta(d) == '5 minutes from now'

    # Check 3 months from now
    d = datetime.datetime(2016, 12, 30, 9, 15)
    assert formatters.chrono.humanize_timedelta(d) == '3 months from now'

    # Check 2 years from now
    d = datetime.datetime(2018, 9, 1, 9, 15)
    assert formatters.chrono.humanize_timedelta(d) == '2 years from now'

    # Check 5 minutes ago
    d = datetime.datetime(2016, 9, 1, 9, 10)
    assert formatters.chrono.humanize_timedelta(d) == '5 minutes ago'

    # Check 3 months ago
    d = datetime.datetime(2016, 6, 1, 9, 15)
    assert formatters.chrono.humanize_timedelta(d) == '3 months ago'

    # Check 2 years ago
    d = datetime.datetime(2014, 9, 1, 9, 15)
    assert formatters.chrono.humanize_timedelta(d) == '2 years ago'

def test_suffix():
    """Return the suffix for a number, e.g. st, nd, rd"""
    assert formatters.chrono.suffix(1) == 'st'
    assert formatters.chrono.suffix(2) == 'nd'
    assert formatters.chrono.suffix(3) == 'rd'
    assert formatters.chrono.suffix(4) == 'th'


from flask import current_app

__all__ = ['path_to_url']


def path_to_url(path):
    """Convert local path to a full URL and return it"""
    if '://' not in path:
        return '{0}://{1}{2}'.format(
            current_app.config['PREFERRED_URL_SCHEME'],
            current_app.config['SERVER_NAME'],
            path
        )

    return path



from bs4 import BeautifulSoup

__all__ = [
    'amped'
]


def amped(html, snippet=None):
    """Convert a HTML string to an AMP HTML string"""

    # Parse the HTML string
    soup = BeautifulSoup(html, 'html.parser')

    # Build a map of images (assets) stored against the snippet
    assets_map = {}
    if snippet:
        if snippet.scope == 'local':
            assets_map = {a[0]: a[1]
                    for a in snippet.local_contents.get('__assets__', [])}
        else:
            contents = snippet.global_snippet.contents
            assets_map = {
                    a[0]: a[1] for a in contents.get('__assets__', [])}

    # Ensure images have width and height attributes
    for tag in soup.select('[data-mh-asset-key]'):
        asset = assets_map.get(tag['data-mh-asset-key'])
        if asset:
            variations = asset.get('variations')
            image_size = variations['image']['core_meta']['image']['size']
            image = tag.find('img')
            if image:
                image['width'] = image_size[0]
                image['height'] = image_size[1]

    # Remove image fixtures
    for tag in soup.select('[data-ce-tag="img-fixture"]'):
        image = tag.find('img')
        if image:
            tag.previous_element.insert_after(image)
            tag.extract()

    # Convert images to amp images
    for tag in soup.select('img'):
        if not tag.get('src'):
            tag.extract()
        tag.name = 'amp-img'
        tag['layout'] = 'responsive'

    # Ensure images within amp-iframes are set as placeholders
    for tag in soup.select('amp-iframe'):
        image = tag.find('amp-img')
        if image:
            image['layout'] = 'fill'
            image['placeholder'] = True
            del image['width']
            del image['height']

    # Remove any inline styles
    for tag in soup.select('style'):
        tag.extract()

    for tag in soup.select('[style]'):
        del tag['style']

    return soup.prettify()



from manhattan import formatters


def test_embed_video_url():
    """Return an embed video URL for a video service"""

    # Vimeo
    vimeo_url = formatters.text.embed_video_url(
        'https://vimeo.com/243498473'
    )
    assert vimeo_url.startswith('https://player.vimeo.com/video/243498473')

    # YouTube
    youtube_url = formatters.text.embed_video_url(
        'https://www.youtube.com/watch?v=7yC2vx_fJy8'
    )
    assert youtube_url.startswith('https://www.youtube.com/embed/7yC2vx_fJy8')

def test_humanize_status():
    """Return a human readable version of a status"""
    assert formatters.text.humanize_status('ACTIVE') == 'Active'
    assert formatters.text.humanize_status('not_available') == 'Not available'

    # Check that custom text formatting is applied when specified
    assert formatters.text.humanize_status('out_of_stock') == 'Out of stock'

    assert formatters.text.humanize_status(
        'invalid_password',
        'INVALID'
        ) == 'INVALID password'

    assert formatters.text.humanize_status(
        'logged_in_as_admin_user',
        'ADMIN',
        'USER'
        ) == 'Logged in as ADMIN USER'

def test_remove_accents():
    """
    Replace any accented characters with there closest non-accented equivalent.
    """
    assert formatters.text.remove_accents('Montréal') == 'Montreal'
    assert formatters.text.remove_accents('Françoise') == 'Francoise'

def test_slugify():
    """Return the given string formatted as a slug"""
    assert formatters.text.slugify('"This is my blog title!"') == 'this-is-my-blog-title'

def test_text_to_html():
    """
    Return a HTML version of the specified text.
    """
    s = '<foo>\nbar\r\n\r\nThis is a link http://www.example.com'.strip()

    # Check output for `inline=True` and `convert_urls=True`
    assert formatters.text.text_to_html(s, True, True) == ('&lt;foo&gt;<br>bar<br>'
            '<br>This is a link <a href="http://www.example.com">'
            'http://www.example.com</a>')

    # Check output for `inline=False` and `convert_urls=True`
    assert formatters.text.text_to_html(s, False, True) == ('<p>&lt;foo&gt;<br>bar</p>'
            '<p>This is a link <a href="http://www.example.com">'
            'http://www.example.com</a></p>')

    # Check output for `inline=True` and `convert_urls=False`
    assert formatters.text.text_to_html(s, True, False) == \
            '&lt;foo&gt;<br>bar<br><br>This is a link http://www.example.com'

    # Check output for `inline=False` and `convert_urls=False`
    assert formatters.text.text_to_html(s, False, False) == ('<p>&lt;foo&gt;'
            '<br>bar</p><p>This is a link http://www.example.com</p>')

def test_upper_first():
    """Return a string with the first character converted to uppercase"""
    assert formatters.text.upper_first('the WWW') == 'The WWW'
    assert formatters.text.upper_first('The WWW') == 'The WWW'
    assert formatters.text.upper_first('the www') == 'The www'

def test_yes_no():
    """Return `'Yes'` if a value coerces to `True`, `'No'` if not."""
    assert formatters.text.yes_no(True) == 'Yes'
    assert formatters.text.yes_no(False) == 'No'
    assert formatters.text.yes_no('foo') == 'Yes'
    assert formatters.text.yes_no(None) == 'No'



from html.parser import HTMLParser
import unicodedata
from urllib.parse import parse_qs, urlencode, urlparse

from jinja2.utils import escape as html_escape, urlize
from slugify import slugify as awesome_slugify
from inflection import humanize, titleize

__all__ = [
    'embed_video_url',
    'humanize_status',
    'remove_accents',
    'remove_markup',
    'slugify',
    'text_to_html',
    'yes_no'
]


def embed_video_url(url):
    """Return the embed video URL for any given youtube or vimeo URL"""

    # Define a table of supported video serving domains
    domains = {
        'www.youtube.com': 'youtube',
        'youtu.be': 'youtube',
        'vimeo.com': 'vimeo',
        'player.vimeo.com': 'vimeo'
    }

    # Parse the domain
    parts = urlparse(url)
    params = parse_qs(parts.query)

    # Check the URL domain is supported
    if parts.netloc not in domains:
        return None

    # Convert the URL
    service = domains[parts.netloc]
    video_id = None
    video_url = None

    if service == 'youtube':

        # Useful default params
        params['enablejsapi'] = 1
        params['rel'] = 0

        if parts.path == '/watch':
            if params.get('v'):
                video_id = params.pop('v')[0]
        else:
            video_id = parts.path[1:]

        if video_id:
            video_url = 'https://www.youtube.com/embed/' + video_id
            if params:
                video_url += '?' + urlencode(params)

    elif service == 'vimeo':
        if parts.path.split('/'):
            video_id = parts.path.split('/')[-1]

        if video_id:
            video_url = 'https://player.vimeo.com/video/' + video_id
            if params:
                video_url += '?' + urlencode(params)

    return video_url

def humanize_status(status, *overrides):
    """
    Return a human readable version of a status with the ability to specify how
    particular words should be output.

    For example:
    humanize_status('out_of_stock') // 'Out of stock'
    humanize_status('invalid_password', 'INVALID') // 'INVALID password'
    humanize_status(
        'logged_in_as_admin_user',
        'ADMIN',
        'USER'
        )  // 'Logged in as ADMIN USER'
    """

    # First humanize the status
    status = humanize(status).lower()

    # For any words which are in the override map find the lowercase equivalent,
    # then replace them with the format of the word the user has specified.
    for s in overrides:
        status = status.replace(s.lower(), s)

    # If the first character is lowercase then make it uppercase
    if status[0].islower():
        status = status[0].upper() + status[1:]

    return status

def remove_accents(s):
    """
    Replace any accented characters with there closest non-accented equivalent.

    This code was taken from the stackoverflow answer here:
    http://stackoverflow.com/questions/517923/what-is-the-best-way-to-remove-accents-in-a-python-unicode-string/517974#517974
    """
    nfkd_form = unicodedata.normalize('NFKD', s)
    return ''.join([c for c in nfkd_form if not unicodedata.combining(c)])

def remove_markup(s):
    """Remove HTML from a string"""
    stripper = _MLStripper()
    stripper.feed(s)
    return stripper.get_data()

def slugify(s):
    """
    Return the given string formatted as a slug
    (https://en.wikipedia.org/wiki/Semantic_URL#slug).
    """
    return awesome_slugify(s, to_lower=True)

def text_to_html(text, inline=False, convert_urls=True):
    """
    Return a HTML version of the specified text.

    If `inline` is True then new-lines will be converted line-breaks (`<br>`s),
    if `inline` is False then single new-lines will be converted to line-breaks
    (`<br>`s) and double new-lines will be converted to paragraphs.

    If `convert_urls` is True then URLs within the text are converted to links,
    if `convert_urls` is False then URLs are not converted.
    """

    # Strip \r from new lines
    text = text.replace('\r\n', '\n')

    if inline:
        # Convert URLs to links
        if convert_urls:
            inline_html = urlize(text)
        else:
            inline_html = str(html_escape(text))

        # Convert newlines to line-breaks / `<br>` tags
        inline_html = inline_html.replace('\n', '<br>')

        return inline_html

    else:
        # Convert the text to a list of paragraphs / <p> tags
        paragraphs = [text_to_html(p.strip(), True, convert_urls) \
            for p in text.split('\n\n') if p.strip()]

        # Join the paragraphs into a body and return it
        return '<p>' + '</p><p>'.join(paragraphs) + '</p>'

def upper_first(s):
    """
    Uppercase the first letter in a string.

    This is useful when converting a string that may already contain uppercase
    characters (for example an acronym) where calling capitalize would
    lowercase existing uppercase characters.
    """

    return s[0].upper() + s[1:]

def yes_no(value):
    """Return `'Yes'` if a value coerces to `True`, `'No'` if not."""
    return 'Yes' if value else 'No'


# -- Utils --

class _MLStripper(HTMLParser):
    """
    A HTML parser for stripping markup from a string.

    This code was taken from the stackoverflow answer here:
    https://stackoverflow.com/a/925630/4657956
    """

    def __init__(self):
        super().__init__()

        self.reset()
        self.strict = False
        self.convert_charrefs= True
        self.fed = []

    def handle_data(self, d):
        self.fed.append(d)

    def get_data(self):
        return ''.join(self.fed)



__all__ = [
    'price'
    ]


def price(amount, symbol='£',  unit_conversion=100):
    """
    Format the given amount as a price, e.g: `'£12.50'`

    Optionally a unit conversion can be specified, the default `100` e.g pence
    to pounds, cents to dollars, etc.

    Optionally a symbol can be specified (e.g `'$'`), we default to the `'£'`.

    NOTE: It is assumed that the price is an integer and not a float or decimal.
    """
    return '{symbol}{amount:,.2f}'.format(
        amount=float(amount) / unit_conversion,
        symbol=symbol
        )


from manhattan import formatters

def test_price():
    """Format an integer as a price"""

    # Check default format is correct
    assert formatters.currency.price(200) == '£2.00'
    assert formatters.currency.price(200000) == '£2,000.00'

    # Check user defined symbol
    assert formatters.currency.price(200, '$') == '$2.00'

    # Check user defined unit conversion
    assert formatters.currency.price(200, '$', 1) == '$200.00'


"""
NOTE: This file provides support for legacy applications, however, we warn
users they should upgrade their imports.
"""

import warnings

from . import chrono
from .chrono import *

from manhattan.utils.chrono import str_to_date, str_to_datetime

__all__ = set(chrono.__all__)
__all__.add('str_to_date')
__all__.add('str_to_datetime')
__all__ = tuple(__all__)


warnings.warn(
    'manhattan.formatters.{dates} should be replaced with chrono',
    FutureWarning
)



import datetime

import humanize
from inflection import ordinal
from manhattan.utils.chrono import (
    localize_datetime,
    now_tz,
    str_to_date,
    str_to_datetime
)
import pytz

__all__ = [
    'humanize_date',
    'humanize_date_range',
    'humanize_datetime',
    'humanize_datetime_tz',
    'humanize_due',
    'humanize_duration',
    'humanize_time',
    'humanize_time_tz',
    'suffix'
]


def humanize_date(d, short=False):
    """
    Return a humanized date, e.g: `'3rd September 2016'`. The function accepts
    a date, datetime or date string (YYYY-MM-DD).
    """

    # Convert strings to date instances
    if isinstance(d, str):
        d = str_to_date(d)

    day = '{0}{1}'.format(d.day, suffix(d.day))

    if short:
        month_year = d.strftime('%b %Y')
    else:
        month_year = d.strftime('%B %Y')

    return '{0} {1}'.format(day, month_year)

def humanize_date_range(d1, d2):
    """
    Return a humanized date range, e.g:

    - `'1st - 3rd September 2016'`
    - `'1st August - 3rd September 2016'`
    - `'1st August 2015 - 3rd September 2016'`
    """
    if d1 == d2:
        return humanize_date(d1)

    day1 = '{0}{1}'.format(d1.day, suffix(d1.day))
    day2 = '{0}{1}'.format(d2.day, suffix(d2.day))

    # Days differ
    if d1.year == d2.year and d1.month == d2.month:
        month_year = d1.strftime('%B %Y')
        return '{0} - {1} {2}'.format(day1, day2, month_year)

    # Months differ
    elif d1.year == d2.year:
        month1 = d1.strftime('%B')
        month2_year = d2.strftime('%B %Y')
        return '{0} {1} - {2} {3}'.format(
            day1,
            month1,
            day2,
            month2_year
            )

    # Years differ
    month1_year = d1.strftime('%B %Y')
    month2_year = d2.strftime('%B %Y')
    return '{0} {1} - {2} {3}'.format(
        day1,
        month1_year,
        day2,
        month2_year
        )

def humanize_datetime(dt, template='{date} at {time}'):
    """
    Return a humanized date/time, e.g: `'3rd September 2016, 17:34'`. The
    function accepts a datetime or date/time string (YYYY-MM-DD HH:MM).

    Optionally the default template can be overidden to modify the returned
    string.
    """
    # Convert strings to datetime instances
    if isinstance(dt, str):
        dt = str_to_datetime(dt)

    return template.format(
        date=humanize_date(dt),
        time=humanize_time(dt)
        )

def humanize_datetime_tz(dt, template='{date} at {time}', tz=None):
    """A version of `humzanize_datetime` that is timezone aware"""
    if isinstance(dt, str):
        dt = str_to_datetime(dt)

    dt = localize_datetime(dt, tz=tz)

    return humanize_datetime(dt, template)

def humanize_due(dt, tz=None):
    """Humanize a due date"""

    dt_tz = localize_datetime(dt, tz=tz)
    d_tz = dt_tz.date()
    now = now_tz()
    today = now.date()

    if dt_tz < now:
        return humanize_timediff(dt)

    if d_tz == today:
        return 'today at ' + dt_tz.strftime('%H:%M')

    if d_tz == (today + datetime.timedelta(days=1)):
        return 'tomorrow at ' + dt_tz.strftime('%H:%M')

    if d_tz < (today + datetime.timedelta(days=7)):
        return '' + dt_tz.strftime('%A at %H:%M')

    if d_tz.year == today.year and d_tz.month == today.month:
        return ''.join([
            dt_tz.strftime('%a, %-d'),
            suffix(d_tz.day),
            dt_tz.strftime(' at %H:%M')
        ])

    if d_tz < date(today.year + 1, today.month, 1):
        return ''.join([
            dt_tz.strftime('%a, %-d'),
            suffix(d_tz.day),
            dt_tz.strftime(' %b at %H:%M')
        ])

    return ''.join([
        dt_tz.strftime('%-d'),
        suffix(d_tz.day),
        dt_tz.strftime(" %b '%y at %H:%M")
    ])

    return str(dt)

def humanize_duration(m, abbr=True):
    """
    Return a humanized duration for a given number of minutes.

    If `abbr` is True the returned string will be of the form `'2hr 5m'`, if
    `abbr` is False then the returned string will be of the form
    `'2 hours 5 minutes'`.
    """
    minutes_str = 'm' if abbr else ' minutes'
    hours_str = 'hr' if abbr else ' hours'

    if m < 60:
        return "{0}{1}".format(m, minutes_str)
    elif m % 60 == 0:
        return "{0}{1}".format(divmod(m, 60)[0], hours_str)
    else:
        h, m = divmod(m, 60)
        return "{0}{2} {1}{3}".format(h, m, hours_str, minutes_str)

def humanize_time(t):
    """Return a humanized a time (24hr), e.g: `'18:30'`"""
    return t.strftime('%H:%M')

def humanize_time_tz(dt, tz=None):
    """
    Return a humanized a time (24hr), e.g: `'18:30'` that is timezone aware.
    """
    if isinstance(dt, str):
        dt = str_to_datetime(dt)

    dt = localize_datetime(dt, tz=tz)

    return humanize_time(dt.time())

def humanize_timediff(dt):
    """
    Return a humanized time difference between now and the specified datetime,
    e.g:

    - `'a second ago'`
    - `'an hour ago'`
    - `'yesterday'`
    """

    if isinstance(dt, datetime.datetime):
        dt = (datetime.datetime.utcnow() - dt).total_seconds()

    return humanize.naturaltime(dt)

def suffix(i):
    """Return the suffix for a number: e.g: `'st'`, `'nd'`, `'rd'`, `'th'`"""
    return ordinal(i)
